@@ -14,16 +14,16 @@ def connect(host:      str,
             password:  str =  'sqream',
             tenant_id: str =  'tenant',
             service:   str =  'sqream',
+            access_token: str = None,
             reconnect_attempts : int = 10,
             reconnect_interval : int = 3,
             query_timeout      : int = 0
-
             ) -> Connection:
     ''' Connect to SQream database '''
 
     conn = Connection(host, port, use_ssl, log=log, is_base_connection=True, reconnect_attempts=reconnect_attempts,
                       reconnect_interval=reconnect_interval, query_timeout=query_timeout)
-    conn.connect_database(database, username, password, tenant_id, service)
+    conn.connect_database(database, username, password, tenant_id, service, access_token)
 
     return conn
 
