@@ -37,4 +37,8 @@ class ProgrammingError(DatabaseError):
 
 
 class NotSupportedError(DatabaseError):
-    pass
\ No newline at end of file
+    pass
+
+
+def is_token_expired(e: str) -> bool:
+    return e is not None and "expired" in e
\ No newline at end of file
